@@ -1,4 +1,4 @@
-/* See LICENSE file for license and copyright information */
+/* SPDX-License-Identifier: Zlib */
 
 #include "config.h"
 
@@ -19,7 +19,7 @@
 
 static void
 cb_window_icon(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(session != NULL && value != NULL);
 
@@ -27,36 +27,37 @@ cb_window_icon(girara_session_t* session, const char* UNUSED(name),
     return;
   }
 
-  char* path        = girara_fix_path(value);
-  GtkWindow* window = GTK_WINDOW(session->gtk.window);
-
-  GError* error = NULL;
-  gtk_window_set_icon_from_file(window, path, &error);
-  free(path);
-
-  if (error == NULL) {
-    return;
-  }
-
-  girara_debug("Failed to load window icon (file): %s", error->message);
-  girara_debug("Trying name instead.");
-  g_error_free(error);
-
-  gtk_window_set_icon_name(window, value);
+  girara_set_window_icon(session, value);
 }
 
 static void
 cb_font(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(session != NULL && value != NULL);
 
   css_template_fill_font(session->private_data->csstemplate, value);
 }
 
+static void
+cb_color(girara_session_t* session, const char* name,
+    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
+{
+  g_return_if_fail(session != NULL && value != NULL);
+
+  const char* str_value = value;
+
+  GdkRGBA color = { 0, 0, 0, 0 };
+  gdk_rgba_parse(&color, str_value);
+
+  char* colorstr = gdk_rgba_to_string(&color);
+  girara_template_set_variable_value(session->private_data->csstemplate, name, colorstr);
+  g_free(colorstr);
+}
+
 static void
 cb_guioptions(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(session != NULL && value != NULL);
 
@@ -67,7 +68,7 @@ cb_guioptions(girara_session_t* session, const char* UNUSED(name),
   bool show_vscrollbar  = false;
 
   /* evaluate input */
-  char* input               = (char*) value;
+  const char* input         = value;
   const size_t input_length = strlen(input);
 
   for (size_t i = 0; i < input_length; i++) {
@@ -110,59 +111,6 @@ cb_guioptions(girara_session_t* session, const char* UNUSED(name),
     GTK_SCROLLED_WINDOW(session->gtk.view), show_hscrollbar, show_vscrollbar);
 }
 
-static void
-cb_scrollbars(girara_session_t* session, const char* name,
-    girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
-{
-  g_return_if_fail(session != NULL && value != NULL);
-
-  const bool val = *(bool*) value;
-
-  char* guioptions = NULL;
-  girara_setting_get(session, "guioptions", &guioptions);
-  g_return_if_fail(guioptions != NULL);
-
-  bool show_hscrollbar = strchr(guioptions, 'h') != NULL;
-  bool show_vscrollbar = strchr(guioptions, 'v') != NULL;
-
-  if (strcmp(name, "show-scrollbars") == 0) {
-    show_hscrollbar = show_vscrollbar = val;
-  } else if (strcmp(name, "show-h-scrollbar") == 0) {
-    show_hscrollbar = val;
-  } else if (strcmp(name, "show-v-scrollbar") == 0) {
-    show_vscrollbar = val;
-  }
-
-  const size_t guioptions_len = strlen(guioptions);
-  char* new_guioptions        = g_try_malloc0(sizeof(char) * (guioptions_len + 3));
-  char* iterator              = new_guioptions;
-  if (new_guioptions == NULL) {
-    g_free(guioptions);
-    return;
-  }
-
-  /* copy everything apart from h and v */
-  for (size_t i = 0; i != guioptions_len; ++i) {
-    if (guioptions[i] != 'h' && guioptions[i] != 'v') {
-      *iterator = guioptions[i];
-      ++iterator;
-    }
-  }
-  g_free(guioptions);
-
-  if (show_hscrollbar == true) {
-    *iterator = 'h';
-    ++iterator;
-  }
-  if (show_vscrollbar == true) {
-    *iterator = 'v';
-    ++iterator;
-  }
-
-  girara_setting_set(session, "guioptions", new_guioptions);
-  g_free(new_guioptions);
-}
-
 void
 girara_config_load_default(girara_session_t* session)
 {
@@ -171,53 +119,47 @@ girara_config_load_default(girara_session_t* session)
   }
 
   /* values */
-  int statusbar_h_padding   = 8;
-  int statusbar_v_padding   = 2;
-  int window_width          = 800;
-  int window_height         = 600;
-  int n_completion_items    = 15;
-  bool show_scrollbars      = false;
+  const int statusbar_h_padding   = 8;
+  const int statusbar_v_padding   = 2;
+  const int window_width          = 800;
+  const int window_height         = 600;
+  const int n_completion_items    = 15;
   girara_mode_t normal_mode = session->modes.normal;
-  bool use_smooth_scroll    = false;
 
   /* other values */
   session->global.autohide_inputbar = true;
 
   /* settings */
   girara_setting_add(session, "font",                     "monospace normal 9", STRING,  FALSE, _("Font"), cb_font, NULL);
-  girara_setting_add(session, "default-fg",               "#DDDDDD",            STRING,  TRUE,  _("Default foreground color"), NULL, NULL);
-  girara_setting_add(session, "default-bg",               "#000000",            STRING,  TRUE,  _("Default background color"), NULL, NULL);
-  girara_setting_add(session, "inputbar-fg",              "#9FBC00",            STRING,  TRUE,  _("Inputbar foreground color"), NULL, NULL);
-  girara_setting_add(session, "inputbar-bg",              "#131313",            STRING,  TRUE,  _("Inputbar background color"), NULL, NULL);
-  girara_setting_add(session, "statusbar-fg",             "#FFFFFF",            STRING,  TRUE,  _("Statusbar foreground color"), NULL, NULL);
-  girara_setting_add(session, "statusbar-bg",             "#000000",            STRING,  TRUE,  _("Statsubar background color"), NULL, NULL);
-  girara_setting_add(session, "completion-fg",            "#DDDDDD",            STRING,  TRUE,  _("Completion foreground color"), NULL, NULL);
-  girara_setting_add(session, "completion-bg",            "#232323",            STRING,  TRUE,  _("Completion background color"), NULL, NULL);
-  girara_setting_add(session, "completion-group-fg",      "#DEDEDE",            STRING,  TRUE,  _("Completion group foreground color"), NULL, NULL);
-  girara_setting_add(session, "completion-group-bg",      "#000000",            STRING,  TRUE,  _("Completion group background color"), NULL, NULL);
-  girara_setting_add(session, "completion-highlight-fg",  "#232323",            STRING,  TRUE,  _("Completion highlight foreground color"), NULL, NULL);
-  girara_setting_add(session, "completion-highlight-bg",  "#9FBC00",            STRING,  TRUE,  _("Completion highlight background color"), NULL, NULL);
-  girara_setting_add(session, "notification-error-fg",    "#FFFFFF",            STRING,  TRUE,  _("Error notification foreground color"), NULL, NULL);
-  girara_setting_add(session, "notification-error-bg",    "#FF1212",            STRING,  TRUE,  _("Error notification background color"), NULL, NULL);
-  girara_setting_add(session, "notification-warning-fg",  "#000000",            STRING,  TRUE,  _("Warning notification foreground color"), NULL, NULL);
-  girara_setting_add(session, "notification-warning-bg",  "#F3F000",            STRING,  TRUE,  _("Warning notifaction background color"), NULL, NULL);
-  girara_setting_add(session, "notification-fg",          "#000000",            STRING,  TRUE,  _("Notification foreground color"), NULL, NULL);
-  girara_setting_add(session, "notification-bg",          "#FFFFFF",            STRING,  TRUE,  _("Notification background color"), NULL, NULL);
-  girara_setting_add(session, "scrollbar-fg",             "#DDDDDD",            STRING,  TRUE,  _("Scrollbar foreground color"), NULL, NULL);
-  girara_setting_add(session, "scrollbar-bg",             "#000000",            STRING,  TRUE,  _("Scrollbar background color"), NULL, NULL);
+  girara_setting_add(session, "default-fg",               "#DDDDDD",            STRING,  FALSE,  _("Default foreground color"), cb_color, NULL);
+  girara_setting_add(session, "default-bg",               "#000000",            STRING,  FALSE,  _("Default background color"), cb_color, NULL);
+  girara_setting_add(session, "inputbar-fg",              "#9FBC00",            STRING,  FALSE,  _("Inputbar foreground color"), cb_color, NULL);
+  girara_setting_add(session, "inputbar-bg",              "#131313",            STRING,  FALSE,  _("Inputbar background color"), cb_color, NULL);
+  girara_setting_add(session, "statusbar-fg",             "#FFFFFF",            STRING,  FALSE,  _("Statusbar foreground color"), cb_color, NULL);
+  girara_setting_add(session, "statusbar-bg",             "#000000",            STRING,  FALSE,  _("Statsubar background color"), cb_color, NULL);
+  girara_setting_add(session, "completion-fg",            "#DDDDDD",            STRING,  FALSE,  _("Completion foreground color"), cb_color, NULL);
+  girara_setting_add(session, "completion-bg",            "#232323",            STRING,  FALSE,  _("Completion background color"), cb_color, NULL);
+  girara_setting_add(session, "completion-group-fg",      "#DEDEDE",            STRING,  FALSE,  _("Completion group foreground color"), cb_color, NULL);
+  girara_setting_add(session, "completion-group-bg",      "#000000",            STRING,  FALSE,  _("Completion group background color"), cb_color, NULL);
+  girara_setting_add(session, "completion-highlight-fg",  "#232323",            STRING,  FALSE,  _("Completion highlight foreground color"), cb_color, NULL);
+  girara_setting_add(session, "completion-highlight-bg",  "#9FBC00",            STRING,  FALSE,  _("Completion highlight background color"), cb_color, NULL);
+  girara_setting_add(session, "notification-error-fg",    "#FFFFFF",            STRING,  FALSE,  _("Error notification foreground color"), cb_color, NULL);
+  girara_setting_add(session, "notification-error-bg",    "#FF1212",            STRING,  FALSE,  _("Error notification background color"), cb_color, NULL);
+  girara_setting_add(session, "notification-warning-fg",  "#000000",            STRING,  FALSE,  _("Warning notification foreground color"), cb_color, NULL);
+  girara_setting_add(session, "notification-warning-bg",  "#F3F000",            STRING,  FALSE,  _("Warning notifaction background color"), cb_color, NULL);
+  girara_setting_add(session, "notification-fg",          "#000000",            STRING,  FALSE,  _("Notification foreground color"), cb_color, NULL);
+  girara_setting_add(session, "notification-bg",          "#FFFFFF",            STRING,  FALSE,  _("Notification background color"), cb_color, NULL);
+  girara_setting_add(session, "scrollbar-fg",             "#DDDDDD",            STRING,  FALSE,  _("Scrollbar foreground color"), cb_color, NULL);
+  girara_setting_add(session, "scrollbar-bg",             "#000000",            STRING,  FALSE,  _("Scrollbar background color"), cb_color, NULL);
   girara_setting_add(session, "word-separator",           " /.-=&#?",           STRING,  TRUE,  NULL, NULL, NULL);
   girara_setting_add(session, "window-width",             &window_width,        INT,     TRUE,  _("Initial window width"), NULL, NULL);
   girara_setting_add(session, "window-height",            &window_height,       INT,     TRUE,  _("Initial window height"), NULL, NULL);
-  girara_setting_add(session, "statusbar-h-padding",      &statusbar_h_padding, INT,     TRUE,  _("Horizontal padding for the status input and notification bars"), NULL, NULL);
-  girara_setting_add(session, "statusbar-v-padding",      &statusbar_v_padding, INT,     TRUE,  _("Vertical padding for the status input and notification bars"), NULL, NULL);
+  girara_setting_add(session, "statusbar-h-padding",      &statusbar_h_padding, INT,     TRUE,  _("Horizontal padding for the status, input, and notification bars"), NULL, NULL);
+  girara_setting_add(session, "statusbar-v-padding",      &statusbar_v_padding, INT,     TRUE,  _("Vertical padding for the status, input, and notification bars"), NULL, NULL);
   girara_setting_add(session, "n-completion-items",       &n_completion_items,  INT,     TRUE,  _("Number of completion items"), NULL, NULL);
-  girara_setting_add(session, "show-scrollbars",          &show_scrollbars,     BOOLEAN, FALSE, _("Show both the horizontal and vertical scrollbars"), cb_scrollbars, NULL);
-  girara_setting_add(session, "show-h-scrollbar",         &show_scrollbars,     BOOLEAN, FALSE, _("Show the horizontal scrollbar"), cb_scrollbars, NULL);
-  girara_setting_add(session, "show-v-scrollbar",         &show_scrollbars,     BOOLEAN, FALSE, _("Show the vertical scrollbar"), cb_scrollbars, NULL);
   girara_setting_add(session, "window-icon",              "",                   STRING,  FALSE, _("Window icon"), cb_window_icon, NULL);
   girara_setting_add(session, "exec-command",             "",                   STRING,  FALSE, _("Command to execute in :exec"), NULL, NULL);
   girara_setting_add(session, "guioptions",               "s",                  STRING,  FALSE, _("Show or hide certain GUI elements"), cb_guioptions, NULL);
-  girara_setting_add(session, "smooth-scroll",            &use_smooth_scroll,   BOOLEAN, TRUE,  _("Enable smooth scrolling and zooming"), NULL, NULL);
 
   /* shortcuts */
   girara_shortcut_add(session, 0,                GDK_KEY_Escape,      NULL, girara_sc_abort,           normal_mode, 0, NULL);
@@ -284,21 +226,25 @@ girara_config_handle_add(girara_session_t* session, const char* identifier, gira
   g_return_val_if_fail(identifier != NULL, false);
 
   girara_session_private_t* session_private = session->private_data;
+  bool found = false;
+
   /* search for existing config handle */
-  GIRARA_LIST_FOREACH(session_private->config.handles, girara_config_handle_t*, iter, data)
-    if (strcmp(data->identifier, identifier) == 0) {
+  GIRARA_LIST_FOREACH_BODY(session_private->config.handles, girara_config_handle_t*, data,
+    if (g_strcmp0(data->identifier, identifier) == 0) {
       data->handle = handle;
-      girara_list_iterator_free(iter);
-      return true;
+      found = true;
+      break;
     }
-  GIRARA_LIST_FOREACH_END(session_private->config.handles, girara_config_handle_t*, iter, data);
+  );
 
-  /* add new config handle */
-  girara_config_handle_t* config_handle = g_slice_new(girara_config_handle_t);
+  if (found == false) {
+    /* add new config handle */
+    girara_config_handle_t* config_handle = g_slice_new(girara_config_handle_t);
 
-  config_handle->identifier = g_strdup(identifier);
-  config_handle->handle     = handle;
-  girara_list_append(session_private->config.handles, config_handle);
+    config_handle->identifier = g_strdup(identifier);
+    config_handle->handle     = handle;
+    girara_list_append(session_private->config.handles, config_handle);
+  }
 
   return true;
 }
@@ -321,6 +267,7 @@ config_parse(girara_session_t* session, const char* path)
   FILE* file = girara_file_open(path, "r");
 
   if (file == NULL) {
+    girara_debug("failed to open config file '%s'", path);
     return false;
   }
 
@@ -334,22 +281,21 @@ config_parse(girara_session_t* session, const char* path)
       continue;
     }
 
-    girara_list_t* argument_list = girara_list_new();
+    girara_list_t* argument_list = girara_list_new2(g_free);
     if (argument_list == NULL) {
       g_free(line);
       fclose(file);
       return false;
     }
 
-    girara_list_set_free_function(argument_list, g_free);
-
     gchar** argv = NULL;
     gint    argc = 0;
 
+    /* parse current line */
     if (g_shell_parse_argv(line, &argc, &argv, NULL) != FALSE) {
       for (int i = 1; i < argc; i++) {
         char* argument = g_strdup(argv[i]);
-        girara_list_append(argument_list, (void*) argument);
+        girara_list_append(argument_list, argument);
       }
     } else {
       girara_list_free(argument_list);
@@ -359,7 +305,7 @@ config_parse(girara_session_t* session, const char* path)
     }
 
     /* include gets a special treatment */
-    if (strcmp(argv[0], "include") == 0) {
+    if (g_strcmp0(argv[0], "include") == 0) {
       if (argc != 2) {
         girara_warning("Could not process line %d in '%s': usage: include path.", line_number, path);
       } else {
@@ -374,7 +320,7 @@ config_parse(girara_session_t* session, const char* path)
           g_free(basename);
         }
 
-        if (strcmp(newpath, path) == 0) {
+        if (g_strcmp0(newpath, path) == 0) {
           girara_warning("Could not process line %d in '%s': trying to include itself.", line_number, path);
         } else {
           girara_debug("Loading config file '%s'.", newpath);
@@ -387,18 +333,16 @@ config_parse(girara_session_t* session, const char* path)
     } else {
       /* search for config handle */
       girara_session_private_t* session_private = session->private_data;
-      girara_config_handle_t* handle = NULL;
-      GIRARA_LIST_FOREACH(session_private->config.handles, girara_config_handle_t*, iter, tmp)
-        handle = tmp;
-        if (strcmp(handle->identifier, argv[0]) == 0) {
+      bool found = false;
+      GIRARA_LIST_FOREACH_BODY(session_private->config.handles, girara_config_handle_t*, handle,
+        if (g_strcmp0(handle->identifier, argv[0]) == 0) {
+          found = true;
           handle->handle(session, argument_list);
           break;
-        } else {
-          handle = NULL;
         }
-      GIRARA_LIST_FOREACH_END(session_private->config.handles, girara_config_handle_t*, iter, tmp);
+      );
 
-      if (handle == NULL) {
+      if (found == false) {
         girara_warning("Could not process line %d in '%s': Unknown handle '%s'", line_number, path, argv[0]);
       }
     }
@@ -416,5 +360,6 @@ config_parse(girara_session_t* session, const char* path)
 void
 girara_config_parse(girara_session_t* session, const char* path)
 {
+  girara_debug("reading configuration file '%s'", path);
   config_parse(session, path);
 }
